#!/bin/bash -xv

# The script is intended to run SExtractor on the five CARS/CFHTLS-D1
# images, to produce a catalogue for BPZ and to produce
# photometric redshifts. Our aim is to have a final LDAC catalogue
# with all SExtractor and photo-z information for galaxies!
#
# I give you a start with the multi-colour catalogue. Probably I did
# not have time to go through all the details of multi-colour catalogue
# creation during the lectues.
#
# Make sure that the BPZPATH variable is set before calling
# this script.

#$1: path to the images
#    we assume that all .cut.conv.fits and corresponding weights
#    are in the directory given here

# We extract sources from the D1 data in dual image mode
# for the five filters:
for FILTER in u g r i z
do
  # get GAIN and MAGZP from image header
  MAGZP=`dfits $1/D1_${FILTER}.V1.6A_Paris.cut.conv.fits |\
         fitsort -d MAGZP | awk '{print $2}'`
  GAIN=`dfits $1/D1_${FILTER}.V1.6A_Paris.cut.conv.fits |\
         fitsort -d GAIN | awk '{print $2}'`
    
  # We ust the 'unconvolved' image for detection (S/N), but measure
  # photometry in the 'convolved' ones.
  # The seeing value below is that from the 'detection' image!
  sex_theli $1/D1_i.V1.6A_Paris.cut.fits,$1/D1_${FILTER}.V1.6A_Paris.cut.conv.fits \
            -c D1_photoz.sex\
            -PARAMETERS_NAME D1_photoz.param \
            -CATALOG_NAME D1_${FILTER}_sex.cat \
            -CATALOG_TYPE FITS_LDAC \
            -WEIGHT_IMAGE \
             $1/D1_i.V1.6A_Paris.cut.weight.fits,$1/D1_${FILTER}.V1.6A_Paris.cut.weight.fits \
            -WEIGHT_TYPE MAP_WEIGHT \
            -FLAG_IMAGE $1/D1_${FILTER}.V1.6A_Paris.cut.flag.fits \
            -FLAG_TYPE MAX \
            -MAG_ZEROPOINT ${MAGZP} \
            -GAIN ${GAIN} \
            -SEEING_FWHM 0.73 \
            -PIXEL_SCALE 0.186

  ldacconv -i D1_${FILTER}_sex.cat -o tmp.cat_$$ -b 1 -c MEGACAM -f ${FILTER}

  # Ok we have a single-colour catalogue now containing quantities
  # from the colour under consideration. Quantities as magnitudes
  # are called MAG_ISO etc. We rename these keys to include
  # colour information in the keynames. This allows us to join
  # these keys to a large multi-colour catalogue below!
  ldacrenkey -i tmp.cat_$$ -o D1_${FILTER}_ldac.cat \
             -t OBJECTS \
             -k FLUX_ISO FLUX_ISO_${FILTER} \
                FLUXERR_ISO FLUXERR_ISO_${FILTER} \
                MAG_ISO MAG_ISO_${FILTER} \
                MAGERR_ISO MAGERR_ISO_${FILTER} \
                MAG_APER MAG_APER_${FILTER} \
                MAGERR_APER MAGERR_APER_${FILTER} \
                IMAFLAGS_ISO IMAFLAGS_ISO_${FILTER} \
                NIMAFLAGS_ISO NIMAFLAGS_ISO_${FILTER}
                
done

# The following ldacjoin commands transfer information on
# magnitudes from the coulour u, g, r and z to the i-band catalogue:
INPUT=D1_i_ldac.cat
i=1

for FILTER in u g r z
do
  ldacjoinkey -i ${INPUT} -o tmp_$i.cat_$$ \
              -t OBJECTS \
              -k MAG_ISO_${FILTER} MAGERR_ISO_${FILTER} \
                 FLUX_ISO_${FILTER} FLUXERR_ISO_${FILTER} \
                 MAG_APER_${FILTER} MAGERR_APER_${FILTER} \
                 IMAFLAGS_ISO_${FILTER} NIMAFLAGS_ISO_${FILTER}\
              -p D1_${FILTER}_ldac.cat

  INPUT=tmp_$i.cat_$$
  i=$(( $i + 1 ))  
done
mv ${INPUT} D1_izrgu.cat
# O.k. The catalogue D1_izrgu.cat has magnitude information from all the
# colours; look at the OBJECTS table of it!

rm tmp*cat_$$

# add masking information to the catalogue:
# In the OBJECTS table there will be a new key MASK having 1
# for all objects falling within a mask (0 otherwise)
ldacaddmask -i D1_izrgu.cat -o D1_izrgu_mask.cat \
            -f $1/D1_V1.6A_Paris_izrgu_pix.reg

#
# filter away objects within masks and 'obvious' stars 
# (SExtractor CLASS_STAR > 0.95):
ldacfilter -i D1_izrgu_mask.cat -o D1_gal.cat \
           -c "((MASK=0))AND(CLASS_STAR<0.95);"

# get a ds9 region files for all galaxies (object display within ds9):

ldactoasc -b -i  D1_gal.cat -t OBJECTS \
          -k Xpos Ypos A B Theta |\
  awk '($1 !~ /#/ ) { printf("ellipse(%d,%d,%d,%d,%f)\n", 
                        int($1), int($2), int(4*$3), int(4*$4), $5)}' \
    > D1_gal_ellipse.reg

# create an ASCII catalogue for BPZ:
# we use an approximate correction for the galactic extinction.
# For the selected D1 region and the used filters 
# mean magnitude corrections are:
# i: 0.057; z: 0.04; r: 0.075; g: 0.10; u: 0.14 
ldactoasc -b -i  D1_gal.cat -t OBJECTS \
          -k SeqNr MAG_ISO_i MAG_ISO_i MAG_ISO_z MAG_ISO_r MAG_ISO_g \
             MAG_ISO_u \
             MAGERR_ISO_i MAGERR_ISO_z MAGERR_ISO_r MAGERR_ISO_g \
             MAGERR_ISO_u |\
  awk '{$2 = $2 - 0.057; $3 = $3 - 0.057; 
        $4 = $4 - 0.040; $5 = $5 - 0.075;
        $6 = $6 - 0.100; $7 = $7 - 0.140; print $0}'> tmp.asc_$$

# see below for an explanation of the file 
# D1_goodfilters.asc; because an awk command below always
# appends to this file we need to delete it if it exists
# already:
test -f D1_goodfilters.asc && rm D1_goodfilters.asc

# the following awk command does the following:
# - filters which have an error larger than 1 get -99 in the
#   magnitudes and 0 in the errors (BPZ convention to not use
#   this filter in the photo-z estimation)
# - it counts the number of filters finally used in BPZ and writes
#   this to a file D1_goodfilters.asc
# - it would be good if you appended this information to your final
#   photometric redshift catalogue. You then can immediately identify
#   objects with problematic photometric coverage.
#
NFILT=5 # we have five filters here
gawk '{goodfilters = 0; 
       for(i = 3; i <= 2 + '${NFILT}'; i++)
       {
         errorcol = i + '${NFILT}' 

         if ($errorcol > 1) {
           $i = -99.0
           $errorcol = 0.0
         } else {
           # we add an error of 0.05 quadratically to account for
           # possible zeropoint inaccuracies:
           $errorcol = sqrt($errorcol * $errorcol + 0.05 * 0.05)
           goodfilters++;  # the number of usable filters for that object
         }
       }
       print goodfilters >> "D1_goodfilters.asc"
       out = ""
       for(i = 1; i <= 2 + 2*'${NFILT}'; i++) {
         out = out $i " "  
       }  
       print out;
      }' tmp.asc_$$ > BPZ_D1.asc

# At this stage we have an ASCII file  BPZ_D1.asc which is suitable
# as inpu for BPZ. To extract the photo-zs, to compare them to
# spectroz-s etc. is your task!
#
# - The column definitions are in D1_izrgu_photoz.columns
# - Use the Capak CWW filters!
# - Use the hdfn_ground prior!


# and bye
exit 0;
