#!/bin/bash

# script to demonstrate basic usage of SExtractor

# command line arguments to the script:

#$1: science image to run SExtractor on (absolute path)
#$2: weight image - gve NONE if you do not
#    want to use a weight (absolute path)

# did we give two command line arguments?
if [ $# -ne 2 ]; then
  echo "first_sex.sh science_image weight_image"
  exit 1;
fi 

# SExtractor takes input parameters from a configuration
# file (here: first_sex.sex; please look at it). 
# They can be overridden via the command line.
# There are several parameters which should to be adapted 
# for each image to obrain menaningful results for certain 
# output parameters:
#
# - MAG_ZEROPOINT: object magnitudes
# - GAIN:          magnitude error estimates 
# - PIXEL_SCALE:   star/galaxy classification
# - SEEING_FWHM:   star/galaxy classification
#(- SATUR_LEVEL:   output flags indicating problems
#                  in the object detection/analysis;
#                  this is dealt with by weights and 
#                  flags in THELI and hence we do not
#                  care about it here)

# before calling SExtractor we extract the parameters above
# from the FITS image header; see the use of 'dfits' 
# (list a FITS header) and 'fitsort' (pick certain items from
# a FITS header)
MAGZP=`dfits ${1}  | fitsort -d MAGZP  | awk '{print $2}'`
GAIN=`dfits ${1}   | fitsort -d GAIN   | awk '{print $2}'`
SEEING=`dfits ${1} | fitsort -d SEEING | awk '{print $2}'`
# In case that images are oriented North Up and East left
# AND if they have CDELT parameters to describe astrometry we 
# can simply extract the pixel scale from the CDELT1 (or CDELT2) 
# element;
# note that CDELT gives scales in degrees, but the
# SExtractor keyword PIXEL_SCALE wants it in arcsec:
PIXSCALE=`dfits ${IMAGE} | fitsort -d CDELT2 | awk '{print $2 * 3600}'`

# o.k. now the SExtractor call; note that the SExtractor I use
# is called 'sex_theli'; we use a simple ASCII catalogue format here:
BASE=`basename $1 .fits`        # for a useful catalogue name
USE_WEIGHT="-WEIGHT_TYPE NONE"
if [ "$2" != "NONE" ]; then
  # The weight type for THELI processed image is ALWAYS
  # MAP_WEIGHT
  USE_WEIGHT="-WEIGHT_TYPE MAP_WEIGHT -WEIGHT_IMAGE $2"
fi

sex_theli ${1} -c first_sex.sex\
          -CATALOG_TYPE ASCII_HEAD \
          -DETECT_MINAREA 3\
          -DETECT_THRESH 2\
          -MAG_ZEROPOINT ${MAGZP} \
          -GAIN ${GAIN} \
          -SEEING_FWHM ${SEEING} \
          -PIXEL_SCALE ${PIXSCALE} \
          -CATALOG_NAME ${BASE}_sex.asc \
          ${USE_WEIGHT}

# create a ds9 region file for easy overplotting of obejcts;
# the regions represent ellipse visualizing the SExtractor
# estimated ellipse parameters. The number is the MAG_AUTO 
# SExtractor estimate. You can use the created region
# file by opening your image in ds9 and 'loading a region file
# in the Regions Menu':
awk '($1 !~ /#/ ) { 
       printf("ellipse(%d,%d,%d,%d,%f) # text = { %.2f }\n", 
              int($2), int($3), 
              int(4 * $7), int(4 * $8), # the "4" scales the
                                        # ellipse for clarity
              $6, $12)
     }' ${BASE}_sex.asc > ${BASE}_sex.reg

# you note the need to know and work with the column numbers
# in ASCII files. One better catalogue format is LDAC which
# is presented in the ldac_demo.sh file.