#!/bin/bash -xv

# This script demonstrates some of the capabilities of the
# LDAC catalog format handling tools.

# Given two of our THELI processed D1 images (e.g.
# D1_i.V1.6A_Paris.cut.fits and D1_g.V1.6A_Paris.cut.fits) we run
# SExtractor on both and merge the resulting catalogues.

# (in real live you probably would use the double image mode for this
# task but we want to demonstrate object catalogue association here;
# probably I did not yet cover this in the lectures - so don't worry
# for the moment :-))

#$1: first image including absolute path
#$2: second image including absolute path
#$3: ds9 mask region file including absolute path
#    (Its name for the D1/CARS Paris data is D1_V1.6A_Paris_izrgu_pix.reg)

if [ $# -ne 3 ]; then
  echo "ldac_demo.sh first_image second_image mask"
  exit 1
fi

# We extract sources from the two images; this is only
# for demonstration purposes and we do not care about weights etc.

i=1 # we use this for output catalogue names; see below
for IMAGE in $1 $2
do
  # The tools 'dfits' and 'fitsort' are very handy to obtain
  # information from header keywords. We need them for SExtractor
  # config parameters below
  MAGZP=`dfits ${IMAGE} | fitsort -d MAGZP | awk '{print $2}'`
  GAIN=`dfits ${IMAGE} | fitsort -d GAIN | awk '{print $2}'`
  SEEING=`dfits ${IMAGE} | fitsort -d SEEING | awk '{print $2}'`

  # In case that images are oriented North Up and East left
  # AND if they have CDELT parameters to describe astrometry we 
  # can simply extract the pixel scale from the CDELT1 (or CDELT2) 
  # element;
  # note that CDELT gives scales in degrees, but the
  # SExtractor keyword PIXEL_SCALE wants it in arcsec:
  PIXSCALE=`dfits ${IMAGE} | fitsort -d CDELT1 | awk '{print -$2 * 3600}'`

  # To produce an LDAC FITS output catalogue with SExtractor the
  # CATALOG_TYPE has to be FITS_LDAC:
  # Note that parameters which are not set here are taken from
  # the configuration file 'ldac_demo.sex': 
  #  
  sex_theli ${IMAGE} -c ldac_demo.sex\
            -CATALOG_TYPE FITS_LDAC \
            -DETECT_MINAREA 5\
            -DETECT_THRESH 3\
            -MAG_ZEROPOINT ${MAGZP} \
            -GAIN ${GAIN} \
            -SEEING_FWHM ${SEEING} \
            -PIXEL_SCALE ${PIXSCALE} \
            -CATALOG_NAME ${i}_sex.cat

  # The following ldacconv is a NECESSARY conversion to
  # use most of the LDAC tools (conversion of tables and keys);
  # just do it!
  ldacconv -i ${i}_sex.cat -o ${i}_ldac.cat -b 1 -c MEGAPRIME -f test

  i=$(( $i + 1 ))
done

# ok, now we run associate; look at the associate.conf!
# Note also that you can merge more than two catalogues!
associate -i 1_ldac.cat 2_ldac.cat \
          -o tmp1.cat tmp2.cat \
          -c associate.conf

# we now make a 'merged catalogue from the two images'. We only
# include positions from the 'first' catalogue and magnitudes from both
# catalogues; look at the 'make_ssc.conf' to see how the configuration
# file for 'make_ssc' is set up:
make_ssc -i tmp1.cat tmp2.cat -o merged.cat -c make_ssc.conf

# If you look at the merged catalogue (table PSSC) 
# you will notice quit some
# lines with zeros for some quantities. These are objects which
# are only present in one of the catalogues. The keys N_00, N_01 ...
# in the PSSC table give information how many objects from the
# individual catalogues contributed. We now filter so that we have
# only objects which are present in both catalogues:
ldacfilter -i merged.cat -o merged_filter.cat -t PSSC \
           -c "(N_00=1)AND(N_01=1);"

# lets calculate a colour term from the two magnitudes and
# name the new column 'colour_m1_m2'; the FLOAT after '-k' says that
# the new key is a floating point number (a good choice nearly
# always):
ldaccalc -i merged_filter.cat -o merged_filter_colour.cat -t PSSC \
         -c "(mag_0-mag_1);" -n colour_m1_m2 "colour term" -k FLOAT   

# now lets make a ds9 region file to visualise the merged objects
# Note the '-b' option to ldactoasc to suppress the image header
# comments:
ldactoasc -b -i merged_filter.cat -t PSSC -k Xpos Ypos |\
  gawk '{printf("circle(%d, %d, 8)\n", int($1), int($2))}' \
  > merged_ds9.reg

# add the A226 mask to the merged catalogue; this will create
# a MASK key with 1 for objects inside a mask:
ldacaddmask -i merged_filter_colour.cat \
            -o merged_filter_colour_mask.cat \
            -t PSSC -f $3

# Just to demonstrate the asctoldac and ldacjoinkey programs;
# we generate random numers between 0 and 1; as many as entries
# in the merged catalogue, convert them to an LDAC table and 
# join them to the merged catalogue (sorry, I could not think of
# anything more 'useful' here):

# get the number of entries in the merged catalogue:
NLINES=`ldactoasc -b -i merged_filter.cat -t PSSC -k SeqNr | wc | awk '{print $1}'`

# now the random numbers:
gawk 'BEGIN {for(i=1; i<= '${NLINES}'; i++) {print rand()}}' > random.asc

# generate a config file for asctoldac on the fly:
{
  echo "COL_NAME = RANDOM"
  echo "COL_TTYPE= FLOAT"
  echo "COL_HTYPE= FLOAT"
  echo 'COL_COMM = "just a random number"'
  echo 'COL_UNIT = ""'
} > asctoldac.conf

# create an LDAC table with the random key and join it to the
# merged catalogue:
asctoldac -i random.asc -o random.cat -t PSSC -c asctoldac.conf

ldacjoinkey -i merged_filter_colour_mask.cat -o merged_final.cat \
            -p random.cat -t PSSC -k RANDOM         

# You noticed one drawback of the LDAC tools: We always create
# new catalogues; I always delete intermediate ones at the end
# and only keep the final one; the following lines are commented
# because you should look at the intermediate ones and understand
# them:
## rm merged_filter_colour_mask.cat
## rm merged_filter_colour.cat
## rm merged_filter.cat
## rm merged.cat
## rm tmp1.cat tmp2.cat
## rm first_sex.cat second_sex.cat