#!/bin/bash -xv

# We extract sources from the simulated Skymaker images produced
# with multicol_simul.sh
# We use the SExtractor double image mode for this.
#
# Result of the script is an LDAC multi-colour catalogue 
# (prefix_izrgu.cat) with an OBJECTS table. It contains magnitude keys 
# MAG_ISO_filter, MAGERR_ISO_filter for the five filters which can
# be used in a photo-z code lateron.
#

PREFIX=$1  # The prefix given as third argument to the multicol_simul.sh
           # script

# We use the i-band as detection image!
#
# Note that for real data you probably will need weight images
# as well!
#

for FILTER in u g r i z
do
  # get the magnitude zero point! Note that images have to be normalised
  # to an effective exposure time of '1s'. Otherwise the magnitude 
  # zeropoint to use
  # in SExtractor is MAGZP+2.5*log(effective EXPTIME) (for SKymaker created
  # data)

  MAGZP=`dfits ${PREFIX}_${FILTER}.fits | fitsort -d MAGZERO | awk '{print $2}'`
  GAIN=`dfits ${PREFIX}_${FILTER}.fits | fitsort -d GAIN | awk '{print $2}'`

  # note that we do not provide a seeing value; we have no stars
  # anyway!  
  sex_theli ${PREFIX}_i.fits,${PREFIX}_${FILTER}.fits -c stuff_photoz.sex \
            -MAG_ZEROPOINT ${MAGZP} \
            -GAIN ${GAIN} \
            -PIXEL_SCALE 0.186 \
            -BACK_TYPE MANUAL -BACK_VALUE 0.0 \
            -CATALOG_NAME ${FILTER}_sex.cat 

  ldacconv -i ${FILTER}_sex.cat -o tmp.cat_$$ \
           -b 1 -c MEGAPRIME -f ${FILTER}

  # Ok we have a single-colour catalogue now containing quantities
  # from the colour under consideration. Quantities as magnitudes
  # are calles MAG_AUTO etc. We rename these keys to include the
  # colour information with the keynames. This allows us to join
  # these keys to a large multi-colour catalogue below!
  ldacrenkey -i tmp.cat_$$ -o ${FILTER}_ldac.cat \
             -t OBJECTS -k MAG_ISO MAG_ISO_${FILTER} \
                MAGERR_ISO MAGERR_ISO_${FILTER} \
                MAG_APER MAG_APER_${FILTER} \
                MAGERR_APER MAGERR_APER_${FILTER}
done

# The following four ldacjoin commands transfer information on
# magnitudes from the coulour u, g, r and z to the i-band catalogue:
ldacjoinkey -i i_ldac.cat -o tmp.cat_$$ \
            -t OBJECTS \
            -k MAG_ISO_z MAGERR_ISO_z MAG_APER_z MAGERR_APER_z \
            -p z_ldac.cat

ldacjoinkey -i tmp.cat_$$ -o tmp1.cat_$$\
            -t OBJECTS \
            -k MAG_ISO_r MAGERR_ISO_r MAG_APER_r MAGERR_APER_r \
            -p r_ldac.cat

ldacjoinkey -i tmp1.cat_$$ -o tmp2.cat_$$\
            -t OBJECTS \
            -k MAG_ISO_g MAGERR_ISO_g MAG_APER_g MAGERR_APER_g \
            -p g_ldac.cat

ldacjoinkey -i tmp2.cat_$$ -o ${PREFIX}_izrgu.cat\
            -t OBJECTS \
            -k MAG_ISO_u MAGERR_ISO_u MAG_APER_u MAGERR_APER_u \
            -p u_ldac.cat

rm tmp*.cat_$$

