#!/bin/bash

# simple awk script to add a constant shear to a 
# stuff 1.11 catalogue; output goes to STDOUT!
#
# we can either use the intrinsic ellipticities from stuff
# or to randomly choose ellipticities with a distribution
# P(e1, e2) \propto (|e|/rho^2)*exp(-(|e|/rho)^2) de1 de2
# where rho is a free parameter and given as fourth argument.
# The smaller rho, the narrower the distribution.

#$1: input stuff catalogue
#$2: input x shear
#$3: input y shear
#$4: rho width parameter of intrinsic 
#    ellipticity distribution 
#    (0=use stuff intrinsic ellip.)

# just for checking purposes we write out a file 'epsilon.txt'
# which contains the observed ellipticies as epsilon1 and epsilon2,
# i.e. as direct estimators of the shear:
test -f epsilon.txt && rm epsilon.txt

# sorry for the following 'awk' program to add ellipticities to
# a stuff catalogue. As so often, a 'one line' awk program grows
# and grows .....
awk '# we need two awk functions for the two ellipticity
     # components; awk only allows a scalar as return value
     #
     # function to calculate first component of observed chi
     function chi1obs ( chi1s, chi2s, g1, g2 ) {
       den = 1.0 + (g1 * g1 + g2 * g2) - \
             2.0 * (- g1 * chi1s - g2 * chi2s);

       chix = (chi1s + 2.0 * g1 + ((g1 * g1 - g2 * g2) * \
               chi1s + 2.0 * g1 * g2 * chi2s)) / den; 

       return chix;
     }

     # function to calculate second component of observed chi
     function chi2obs ( chi1s, chi2s, g1, g2 ) {
       den = 1.0 + (g1 * g1 + g2 * g2) - \
             2.0 * (- g1 * chi1s - g2 * chi2s);

       chiy = (chi2s + 2.0 * g2 + (-(g1 * g1 - g2 * g2) * \
               chi2s + 2.0 * g1 * g2 * chi1s)) / den; 

       return chiy;
     }

     { # stuff gives us intrinsic axis ratios and position angles -
       # for disks and buldges. For both galaxy components
       # we calculate the corresponding observed quantities with
       # the chi-ellipticity formula.
       #
       # if the fourth script argument is zero we use the
       # intrinsic stuff ellipticity; otherwise we choose
       # random ellipticities according to the distribution given
       # above
       if ('$4' < 1.0e-06) {
         # first transform stuff disk position angle and axis ratios
         # to chi1 and chi2 components:
         thetad = $11;   # position angle
         ratd   = $10;   # ellipticity axis ratio

         # absolute value of ellipticity
         chiabsd = (1. - ratd * ratd) / (1. + ratd * ratd);

         # instrinsic chi ellipticity components
         chi1s = chiabsd * cos(2. * 3.14159 / 180. * thetad);
         chi2s = chiabsd * sin(2. * 3.14159 / 180. * thetad);
       } else {
         # o.k. we need a random ellipticity
         # make sure we do not get ellipticities larger than 1:
         epsabsd = 2.0;  
         while(epsabsd > 1.0) {
           # the following chooses a random ellipticity modulus
           # according to the formula above; have a look to Numerical
           # Recipes if you are interested!
           epsabsd = '$4' * sqrt(-log(1.0 - rand()));
         }
         # random, position angle
         thetad = 6.283185 * rand();
  
         # transform epsilon ellipticity in chi ellipticity:
         chiabsd = (2.0 * epsabsd) / (1.0 + (epsabsd * epsabsd));
  
         chi1s = chiabsd * cos(2.0 * thetad);
         chi2s = chiabsd * sin(2.0 * thetad);
       }

       # observed disk ellipticity components:
       chi1d = chi1obs(chi1s, chi2s, '$2', '$3'); 
       chi2d = chi2obs(chi1s, chi2s, '$2', '$3'); 

       # recalculate theta and disk axisratio from the
       # sheared ellipticities - we need
       # to put them back to a stuff catalogue for skymaker:
       thetad = 180. * 0.5 * \
                atan2(chi2d, (chi1d + 0.0001)) / 3.14159;
       chiabsd = sqrt(chi1d * chi1d + chi2d * chi2d);
       ratd = sqrt((1. - chiabsd) / (1. + chiabsd));
        
       $10 = ratd;
       $11 = thetad;

       # now the same for the buldge; the intrinsic ellipticity
       # for the buldge is assumed to be zero:
       chi1s = 0;
       chi2s = 0;
       chi1b = chi1obs(chi1s, chi2s, '$2', '$3'); 
       chi2b = chi2obs(chi1s, chi2s, '$2', '$3'); 
       thetab = 180. * 0.5 * \
                atan2(chi2b, (chi1b + 0.0001)) / 3.14159;
       chiabsb = sqrt(chi1b * chi1b + chi2b * chi2b);
       ratb = sqrt((1. - chiabsb) / (1. + chiabsb));

       # calculate the epsilon quantity and store it to a file
       # "epsilon.txt" (good for quick checks)
       epsilonabsd = chiabsd / (1. + sqrt(1. - chiabsd * chiabsd));
       print epsilonabsd * cos(2. * thetad * 3.14159 / 180.), \
             epsilonabsd * sin(2. * thetad * 3.14159 / 180.) >> "epsilon.txt";

       $7 = ratb;       
       $8 = thetab;

       # just print the modified line to STDOUT
       print $0;              
     }' $1
