#!/bin/bash 

# This script creates five-band multi-colour simulations
# with SkyMaker. The parameters in this script are tuned 
# to result in data with about the same characteristics as
# our CFHTLS/CARS data. Simulated is the field-galaxy 
# population, NOT clusters or other 'peculiar' objects.

# The results of the script after successful execution 
# are as follows; you will find all files in the directory
# where you called this script:
#
# - ASCII output files from stuff:
#   Names are prefix_filter.asc 
#   where prefix is the thirs command line argument to this 
#   script and filter one of the five strings u, g, r, i 
#   or z.
#
# - LDAC catalogues created from the ASCII stuff files; their
#   names are: prefix_filter_ldac.cat
#
# - Skymaker images created from the ASCII stuff catalogues;
#   their names are:  prefix_filter.fits  
#   IMPORTANT NOTE: After the Skymaker Run the images are
#   rescaled to an exposure time of 1s; the magnitude zeropoint
#   to be given to SExtractor is that you find in the image
#   headers of the simulates images! 

# Before using this script edit the location of some
# program variables below!

# Besids Skymaker and Stuff this script assumes a present
# installation of the THELI pipeline!

#
# command line arguments to this script:
#

#$1: size_x                 # X-dimension (pixels) of simulated images
#$2: size_y                 # Y-simension (pixels) of simulated images
#$3: simulation name prefix # names of resulting files will be $3_....

# check the number of given command line arguments:
if [ $# -ne 3 ]; then
  echo "Syntax: multicol_simul.sh dim_x dim_y nameprefix"
  exit 1;
fi

# Put here the Paths to your skymaker and stuff installations:
P_SKY=/aibn162_1/terben/skymaker-3.3.3/src/sky
STUFFDIR=/aibn162_1/terben/stuff-1.11
P_STUFF=/aibn162_1/terben/stuff-1.11/src/stuff
P_REPLACEKEY=/aibn162_1/terben/software/ldacpipeline/bin/Linux_64/replacekey

CONFIG=.
export CONFIG

# First we run Stuff to produce five-band multi-colour catalogues
# for the MEGAPRIME filters u, g, r, i and z:

# simulation specific data:
SIZEX=$1
SIZEY=$2
PIXSCALE=0.186       # MEGAPRIME@CFHT

# have a look at the MEGAPRIME_multicol.stuff configuration file, too!
${P_STUFF} -c ${CONFIG}/MEGAPRIME_multicol.stuff \
           -CATALOG_NAME $3_u.asc,$3_g.asc,$3_r.asc,$3_i.asc,$3_z.asc\
           -IMAGE_WIDTH ${SIZEX} \
           -IMAGE_HEIGHT ${SIZEY} \
           -PIXEL_SIZE ${PIXSCALE} \
           -SEED_GALAXIES 100 \
           -DATA_DIRECTORY ${STUFFDIR}/data

# create immediately LDAC catalogues from the ASCII stuff data
# (for later merging with extracted catalogues for instance)
for FILTER in u g r i z
do
  # The last four quantities (2.0, 2.0, 0.0, 0) are for object
  # matching in pixel coordinates and a flag (0=everything ok)
  awk '{print NR, $0, 2.0, 2.0, 0.0, 0}' $3_${FILTER}.asc > tmp_$$.asc

  asctoldac -i tmp_$$.asc -o $3_${FILTER}_ldac.cat \
            -c asctoldac_stuff_1.11.conf

  rm tmp_$$.asc
done

# some parameters for the skymaker simulation. The magnitude
# zeropoints and total exposure times directly come from our 
# D1 data. The sky background magnitudes and the central wavelengths
# of the filter come from the CFHT/MEGAPRIME Web side
# (see http://www.cfht.hawaii.edu/Instruments/Imaging/Megacam/generalinformation.html). 
FILTER=([1]="u" [2]="g" [3]="r" [4]="i" [5]="z")
MAGZEROPOINT=([1]=25.3 [2]=26.4 [3]=25.9 [4]=25.7 [5]=24.8)
BACKMAG=([1]=22.1 [2]=22.0 [3]=21.4 [4]=20.4 [5]=19.6)
CWL=([1]=0.38 [2]=0.48 [3]=0.62 [4]=0.76 [5]=0.88)
TEXPTIME=([1]=51500 [2]=44800 [3]=100000 [4]=188000 [5]=106000)
#SEEING=([1]=0.95 [2]=0.94 [3]=0.79 [4]=0.82 [5]=0.74) # real values in D1 data
SEEING=([1]=0.9 [2]=0.9 [3]=0.9 [4]=0.9 [5]=0.9) # for simplicity in first
                                                 # photo-z tests (same seeing!)

# Now create SkyMaker FITS images from the Stuff catalogues; Skymaker
# only allows the creation of one image at a time (unless stuff);
# hence we need to do the SkyMaker calls for the five filters in a
# loop:

i=1
while [ ${i} -le 5 ]
do
  # We want to reach the total exposure time of many thounsands of seconds
  # by chunks of 500s (why?). To simulate this we need to multiply the gain
  # (1.62 for MEGAPRIME) and the fullwell capacity (150000 for MEGAPRIME) 
  # accordingly   
  GAIN=`gawk 'BEGIN {print '${TEXPTIME[$i]}' * 1.62 / 500}'`
  FULLWELL=`gawk 'BEGIN {print '${TEXPTIME[$i]}' * 150000 / 500}'`

  # Note especially the setting '-STARCOUNT_ZP  0.0' which creates a
  # simulation WITHOUT stars; stuff does not yet include the creation
  # of stars and Skymaker would add each time stars at random positions
  # with random magnitudes
  # also if we fix the random position issue (easy to do) the stars would
  # not have 'correct colours' for a multi-colour observation!
  ${P_SKY} $3_${FILTER[$i]}.asc \
           -c ${CONFIG}/field.sky \
           -IMAGE_NAME $3_${FILTER[$i]}_skyorig.fits \
           -IMAGE_SIZE ${SIZEX},${SIZEY} \
           -PIXEL_SIZE ${PIXSCALE} \
           -MAG_ZEROPOINT ${MAGZEROPOINT[$i]} \
           -BACK_MAG ${BACKMAG[$i]} \
           -WAVELENGTH ${CWL[$i]} \
           -GAIN ${GAIN} \
           -WELL_CAPACITY ${FULLWELL} \
           -SEEING_FWHM ${SEEING[$i]} \
           -EXPOSURE_TIME 500 \
           -STARCOUNT_ZP  0.0

  # The result so far has an effective exposure time of 500s!
  # Note that if you wanted to use it in SExtractor and obtain
  # magnitudes you would need to put (MAG_ZEROPOINT + 
  # 2.5*log(effective EXPOSURE TIME)) as corresponding config parameter.
  #
  # We now subtract the mode (the skybackground) and divide by the
  # exposure time. By this we create an image with units ADU/s
  # and an effective exposure time of 1s!!

  # The skybackground value is estimated with the THELI 'stats' program giving
  # information on the pixel statistics in the images:
  SKYBACK=`stats < $3_${FILTER[$i]}_skyorig.fits | awk '$1=="mode" {print $3}'`
  ic '%1 '${SKYBACK}' - 500 /' $3_${FILTER[$i]}_skyorig.fits > \
    $3_${FILTER[$i]}.fits  

  # We also need to adapt the gain keyword accordingly:
  GAIN="`dfits $3_${FILTER[$i]}.fits | fitsort -d GAIN | awk '{printf("%20f", $2*500)}'`"
  ${P_REPLACEKEY} $3_${FILTER[$i]}.fits "GAIN    = ${GAIN}" GAIN

  rm $3_${FILTER[$i]}_skyorig.fits

  i=$(( $i + 1 ))
done

