/*
                                  galaxies.c

*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*
*       Part of:        Stuff
*
*       Author:         E.BERTIN (IAP)
*
*       Contents:       Functions dealing more specifically with galaxies.
*
*       Last modify:    26/01/2005
*
*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
*/

#ifdef HAVE_CONFIG_H
#include	"config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "define.h"
#include "globals.h"
#include "cosmo.h"
#include "lf.h"
#include "galaxies.h"
#include "prefs.h"
#include "random.h"
#include "sed.h"


/********************************* gal_init **********************************/
/*
Create type-dependent galaxy structures.
*/
galstruct	*gal_init(sedstruct *sed, int nsed)
  {
   galstruct	*gal;
   lfstruct	*lf;
   double	bt;
   int		g,n, ngal;

  ngal = prefs.gal_ntype;
/* Allocate memory for the galaxy array */
  QCALLOC(gal, galstruct, ngal);

/* Go through each galaxy type */
  for (g=0; g<ngal; g++)
    {
    gal[g].bt = bt = prefs.gal_bt[g];
/*-- Pre-compute magnitude-offsets for bulge and disk */
    gal[g].bdm = bt>0.0? -2.5*log10(bt) : 100.0;
    gal[g].ddm = bt<1.0? -2.5*log10(1.0-bt) : 100.0;
/*-- Initialize the luminosity function */
    QMALLOC(lf, lfstruct, 1);
    gal[g].lf = lf;
    lf->phistar = prefs.lf_phistar[g]*H*H*H/(MPC*MPC*MPC);
/*-- Subtract from M* the average disk extinction to get the face-on value */
    lf->mstar = prefs.lf_mstar[g] + deltaMH
		- 2.5*log10(1.0+(1.0-bt)*DEXP(0.12041*prefs.gal_iextinc[g]));
    lf->alpha = prefs.lf_alpha[g];
    lf->mabsmax = prefs.lf_mlim[1]+deltaMH;
    lf->mabsmin = prefs.lf_mlim[0]+deltaMH;
    gal[g].beta = prefs.gal_beta;
    gal[g].disk_rstar = prefs.gal_rstar*PC/H;
/*-- Spectral Energy Distributions (SEDs) */
/*-- Bulge component */
    if ((n=prefs.gal_bsedno[g]-1) >= nsed)
      error(EXIT_FAILURE, "*Error*: no such Bulge SED-index", "");
    gal[g].bsed = sed + n;
/*-- Bulge component */
    if ((n=prefs.gal_dsedno[g]-1) >= nsed)
      error(EXIT_FAILURE, "*Error*: no such Disk SED-index", "");
    gal[g].dsed = sed + n;
    }

  return gal;
  }


/********************************* gal_end ***********************************/
/*
Terminate galaxy structures.
*/
void	gal_end(galstruct *gal, int ngal)
  {
   int	i;

  for (i=0; i<ngal; i++)
    free(gal[i].lf);

  free(gal);

  return;
  }


/******************************* gal_bulgesize *******************************/
/*
Returns the average bulge size (in m) as a function of the absolute magnitude.
From Fig. 7 in Binggeli et al. (1984).
*/
double gal_bulgesize(double mabs)
  {
/* Subtract the log10(re) = f(Mabs) knee from the absolute magnitude */
  mabs -= GAL_BULGEMKNEE + deltaMH;

  return GAL_BULGERKNEE*DEXP(-(mabs<0.0? 0.3:0.1)*mabs)*PC/H;
  }


/******************************* gal_bulgeflat *******************************/
/*
Returns the (random) apparent bulge flattening as a function of
cos(inclination) (Sandage et al. 1970). Warning: i is here in quadrature with
the original Sandage definition.
*/
double gal_bulgeflat(double cosi)
  {
   double	q;

/* Distribution of intrinsic flattening */
  q = GAL_BULGEQMEAN + random_gauss(GAL_BULGEQSIG);
  if (q>1.0)
    q = 1.0;

  return sqrt((q*q-1.0)*(1.0-cosi*cosi) + 1.0);
  }


/******************************* gal_disksize ********************************/
/*
Returns the (random) disk scale-size (in m) as a function of the absolute
magnitude (de Jong & Lacey 1998).
*/
double gal_disksize(galstruct *gal, double mabs)
  {
/* A log-normal distribution */
  return gal->disk_rstar*exp(0.921*(mabs-gal->lf->mstar)*(2.0/gal->beta-1.0)
	+random_gauss(GAL_DISKSIGMAL))/(1.67835*H);
  }


/********************************* gal_cosi **********************************/
/*
Returns the (random) cos(inclination).
*/
double gal_cosi(void)
  {
/* Flat angular distribution */
  return cos(random_double()*GAL_MAXINCLIN*DEG);
  }

