
/*---------------------------------------------------------------------------
   
   File name 	:	wfi_split.c
   Author 		:	N. Devillard
   Created on	:	18 May 2000
   Description	:	WFI frame splitter

 ---------------------------------------------------------------------------*/

/*
	$Id: mefsplit.c,v 1.18 2006/08/04 22:51:40 thomas Exp $
	$Author: thomas $
	$Date: 2006/08/04 22:51:40 $
	$Revision: 1.18 $
*/

/*---------------------------------------------------------------------------
   								Includes
 ---------------------------------------------------------------------------*/

/* 
   05.09.03:
   changes to include the OBJECT keyword in the new headers

   14.09.03:
   included set_verbose(1) to get comment messages printed by
   e_comment etc. The messages previously appeared on Linux SuSE 
   but not on Sun and Linux Debian 

   13.10.03:
   included possibility to flip frames in x and/or y

   20.07.2004:
   I added the '-d' option (dump configuration default values) 
   to the command line arguments

   30.11.2004:
   corrected two bugs in the sanity checking of command
   line arguments.

   23.01.2005:
   I changed the location of the actual prefstruct definition
   to avoid double existence of it during the link process.

   27.01.2005:
   added global version number information to the splitted images

   18.02.2005:
   I increased the minor version number due to HISTORY output
   changes in wfip_lib.c

   24.02.2005:
   I increased the minor version number due to bug fixes
   in wfip_lib.c

   19.04.2005:
   - I introduced a matrix transformation for images allowing
     arbitrary flips and rotations by a multiple of 90 degrees.
     (This replaces the old flipping possibility in x and y)
   - I increased the version number to 2.0
   - I added USAGE information text.
   - New mode 'no header update': We can split images with
     no header updateing/remapping. In this case the old headers
     (main and extension) are preserved. In this case we do not
     check the integrity of given header keywords!

   26.04.2005:
   I increased the minor version number due to a bug fix
   in wfip_lib.c.

   05.07.2005:
   I increased the minor version number due to a major bug fix
   in wfip_lib.c.

   11.10.2005:
   I included the possibility to give the actual IMAGEID under that
   a certain extension should be saved; increase of middle version
   number.

   24.10.2005:
   Bug fix concerning the treatment of IMAGEID parameter ->
   increase of minor version number.

   23.11.2005:
   I introduced the OUTPUT_DIR command
   line argument. It allows to explicitely specify
   the output directory for splitted data.

   16.05.2006:
   I updated the error message on inconsistent parameters.

   05.08.2006:
   I included the possibility to provide header information for
   only ONE image if a single extension has to extracted
   (command line option '-s').
*/
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "global_vers.h"
#include "eclipse.h"
#include "wfip_lib.h"
#include "prefs_split.h"

#define __PROGRAM__ "mefsplit: Mosaic camera image splitter (MEF files)"
#define __VERS__ "2.2.0"

#define __USAGE__ "\
USAGE:\n\
   mefsplit image [options]\n\n\
OPTIONS:\n\
-h      print this and leave\n\
-d      dump standard configuration file and leave\n\
-c      name of the configuration file\n\n\
-x      extension to extract from MEF file\n\
        (default is 0, i.e. extract all extensions)\n\
-s      Provide header information ONLY for the extension\n\
        to be splitted (in case '-x' is set)\n\
-u      NO update of original image headers\n\
all options in the configuration file can be\n\
overriden on the command line via:\n\
-OPTIONNAME OPTIONVALUE\n\n\
DESCRIPTION:\n\
The program splits a Multi Extension FITS File (MEF)\n\
into individual chips. By default, the output consists of\n\
all extensions from the MEF file. Output filenames are \n\
BASE_i.fits, where 'i' is the number of the extension \n\
(starting from 1) and the original filename is BASE.fits\n\
With command line option '-x' only a specified extension\n\
is selected. Headers of the original images are replaced\n\
by new headers containing only keywords to uniquely determine\n\
astrometric and photometric properties: CRPIXi, CDij, CRVALi\n\
EXPTIME, EQUINOX, AIRMASS, GABODSID, FILTER, OBJECT (see the\n\
GaBoDS pipeline documentation). To this end the values for\n\
these quantities have to be provided on the command line.\n\
Call 'mefsplit -d' for an example of dummy values for WFI@ESO 2.2m.\n\
Additionally, for each extension a matrix Mij has to be given.\n\
Chips are transformed according to the matrix before being\n\
written to disk. The matrix's elements have to be 0,+1,-1 and\n\
det(M)=+1 or -1. This allows arbitraty flips and rotations by\n\
angles of 90 degrees. Where it is important, images are first\n\
rotated and then flipped.\n\
During this process, new astrometric quantities (CRPIXi, CDij)\n\
are calculated out of the values given on the command line.\n\
We strongly recommend to bring all chips to the SAME sky\n\
orientation during this process.\n\
With the option '-u' files can be splitted without header\n\
update. In this case, the old headers are completely preserved\n\
for each extension.\n\n\
AUTHOR:\n\
  Thomas Erben (terben@astro.uni-bonn.de)\n\n\
  The code is largely based on the eclipse library\n\
  (see http://www.eso.org/eclipse) and on sources\n\
  from Emmanuel Bertin for the handling of configuration\n\
  options.\n"

static void usage(void) ;
prefstruct	prefs;

/*---------------------------------------------------------------------------
									Main	
 ---------------------------------------------------------------------------*/

int main(int argc, char *argv[])
{
    int             a, narg, nim ;
    int             i;
    char	    name_i[FILENAMESZ] ;
    char	    name_o[FILENAMESZ] ;
    int		    xtnum ;
    char	    **argkey, **argval, *str;
    int             headerupdate = 1; /* should headers of original images
                                         be replaced ? */
    int             singleimagemode = 0; /* If we extract only one extension
                                            from an MEF file, do we have to
                                            provide header info. for all the
                                            extensions nevertheless (0=YES) */
    int             imageidcheck[MAXCHIPS];
    
    xtnum = narg = nim = 0 ;

    QMALLOC(argkey, char *, argc);
    QMALLOC(argval, char *, argc);

    /* initialisations */
    strcpy(prefs.prefs_name, "default.split");
    prefs.nimageid = 0;

    /* read command line arguments */

    if(argc < 2)
    {
	usage();
    }
 
    for (a=1; a<argc; a++)
    {
      if (argv[a][0] == '-')
      {
        if (strlen(argv[a])<3)
          switch((int)tolower((int)argv[a][1]))
          {
  /*-------- Config filename */
	      case 'x':	if(a<(argc-1))
	                {
                          xtnum = atoi(argv[++a]); 
			}
	                else
		        {
			    usage();  
			}
  			break;
            case 'd':	dumpprefs();
         		exit(0);
  			break;
            case 'h':	usage(); 
  			break;
            case 'u':   headerupdate = 0; 
  			break;
            case 's':   singleimagemode = 1; 
  			break;
            case 'c':	if(a<(argc-1))
	                {
                            strncpy(prefs.prefs_name, argv[++a], MAXCHAR-1); 
			}
	                else
		        {
			    usage();  
			}
  			break;
            default:	usage();
		        break;
          }
        else
        {
  /*------ Config parameters */
          argkey[narg] = &argv[a][1];
          argval[narg++] = argv[++a];
        }       
      }
      else
      {
  /*---- The input image filename(s) */
        for(; (a<argc) && (*argv[a]!='-'); a++)
          for (str=NULL;(str=strtok(str?NULL:argv[a], notokstr)); nim++)
            if ((nim<MAXIMAGE))
              prefs.image_name[nim] = str;
            else
              error(EXIT_FAILURE, "*Error*: Too many input images: ", str);
        prefs.nimage_name = nim;
        a--;
      }
    }

    readprefs(prefs.prefs_name, argkey, argval, narg);

    free(argkey);
    free(argval);

    /* Initialize eclipse environment */
    eclipse_init() ;
    set_verbose(1); /* print comment messages */

    /* sanity check; we do not care if NO header update
       is performed */
    if(headerupdate != 0)
    {
      /* set default ordering of chips, i.e. the
         nth chip in the MEF file gets IMAGEID n
         if the user did not supply the ordering
       */
       if(prefs.nimageid == 0)
       {
	 prefs.nimageid = prefs.ncrpix1;

	 /* if we extract only a single extension,
            the default IMAGEID is equivalent to the
            extension number */
 	 if (xtnum != 0)
	 {
	   prefs.imageid[0]=xtnum;
	 }
	 else
	 {

	   for(i=1; i<= prefs.nimageid; i++)
	   {
	     prefs.imageid[i-1]=i;
	   }
	 }
       }
       else
       {
	 for(i=0; i < MAXCHIPS; i++)
	 {
	   imageidcheck[i] = 0;
	 }

	 for(i=0; i<prefs.nimageid; i++)
	 {
	   if(imageidcheck[prefs.imageid[i]-1] != 0)
	   {
             error(EXIT_FAILURE, "Double IMAGEID !!\n", ""); 
	   }
	   imageidcheck[prefs.imageid[i]-1] = 1;
	 }
       }

       if((prefs.ncrpix1 != prefs.ncrpix2) ||
          (prefs.ncrpix1 != prefs.ncd11) ||
          (prefs.ncrpix1 != prefs.ncd22) ||
          (prefs.ncrpix1 != prefs.ncd12) ||
          (prefs.ncrpix1 != prefs.ncd21) ||
        (prefs.ncrpix1 != prefs.nm11) ||
  	(prefs.ncrpix1 != prefs.nm12) ||
  	(prefs.ncrpix1 != prefs.nm21) ||
  	(prefs.ncrpix1 != prefs.nm22) ||
  	(prefs.ncrpix1 != prefs.nimageid))
  	{
           error(EXIT_FAILURE, "Inconsistent number of elements in crpix1, crpix2, cd11, cd22, cd12, cd21, M11, M12, M21, M22\n", ""); 
  	}
    }

    /* do the real stuff */
    for(a=0; a < prefs.nimage_name; a++)
    { 
	/* Get input and output names */

	strcpy(name_i, prefs.image_name[a]);

	if(strcmp(prefs.outputdir, "") == 0)
	{
	  strcpy(name_o, get_rootname(name_i));
	}
	else
	{
	  sprintf(name_o, "%s/%s", prefs.outputdir, get_rootname(get_basename(name_i)));
	}

	wfi_mysplit(name_i, name_o, xtnum, 
                    headerupdate, singleimagemode,
                    prefs.crpix1,
                    prefs.crpix2,
                    prefs.cd11,
                    prefs.cd22,
                    prefs.cd12,
                    prefs.cd21,
		    prefs.m11,
		    prefs.m22,
		    prefs.m12,
		    prefs.m21,
		    prefs.imageid,
		    prefs.ncrpix1,
                    prefs.crval1,
                    prefs.crval2,
                    prefs.exptime,
                    prefs.equinox,
                    prefs.airmass,
		    prefs.gabodsid,
                    prefs.filter,
                    prefs.object,
                    __PIPEVERS__);
    }

    if (debug_active())
		xmemory_status() ;
    return(0) ;
} 



/*
 * This function only gives the usage for the program
 */

static void usage(void)
{
    printf("%s \n(Program Version:%s; %s)\n\n%s", __PROGRAM__, 
           __VERS__, __PIPEVERS__, __USAGE__) ;
    exit(1) ;
} 
