#!/usr/bin/env bash

# Main installation script for the THELI pipeline

# IMPORTANT:
#
# The script checks machine type, updates pipeline
# scripts (progs.ini; pipe_env ...)
# and compiles pipeline sources
# the script needs the source file 'endian.c' in the
# directory it is called. Modify the paths listed below
# before running this script.

#
# Modes of the script
# 
# ./install.sh -m "TEST"
# tests the machine type and gives information whether
# you can compile sources without problems or whether
# you have to create new Makefiles (see the documentation
# for more details)
#
# ./install.sh -m "ALL"
# updates paths in all pipeline scripts and compiles
# pipeline sources
#
# ./install.sh -m "UPDATE"
# only updates paths in pipeline scripts
#
# ./install.sh -m "COMPILE"
# only compiles pipeline sources

#
# HISTORY COMMENTS:
# 
# 09.03.2006:
# - I removed the utilities scripts subdirectory from
#   the script updates. It contains no files needed in
#   the official THELI version.
# - I included MAC/Darwin systems to the known architectures.
#
# 14.03.2006:
# I changed the installation scheme for ASTROMETRIX: instead
# of overriding the original script files with modified ones,
# I keep the modified, renamed, ones in the pipeline structure
# and add the corresponding directory the PERL5LIB variable. 
# This allows us to directly call the modified ASTROMETRIX files 
# with the perl interpreter.
#
# 27.04.2006:
# I adapted the location of 'global_vers.h' (it is now in
# ../ldactools/library and no longer in ../library)
#
# 01.05.2006:
# The pipeline version for the external version is
# 'THELI Pipeline Version: XXXX'. This allows a distinction
# to the internal sources.
#
# 15.05.2006:
# - I included a simple check for the presence of necessary, external 
#   programs defined in this script.
# - The script now prints some description when being invoked.
#
# 17.05.2006:
# - I changed slightly the welcome message
# - increase of the minor pipeline version number. 
# - I improved messages during the installation process.
# - The handling of the test for the availabbility of external
#   UNIX programs is more stable to account for architecture dependent 
#   behaviours of the 'which' program.
#
# 14.06.2006:
# - The availability of the external 'sm' program is checked via
#   a 'which' command now. 
# - Increase of minor pipeline version number.
#
# 30.08.2006:
# Increase of minor pipeline version number.
#
# 05.11.2006:
# - If the program 'sm' is not present, the variable P_SM
#   is set to an empty string. Without this the value could
#   be quite undefined. This avoids potential problems
#   in the 'sed' command which modifies the progs.ini file.
# - In the 'sed' command which modifies the progs.ini file 
#   I removed lines refering to IRAF.
#
# 06.11.2006:
# The version number goes to 0.12.0
#
# 18.09.2007:
# I included code for the possibility to use scamp within
# THELI. The script documentation was updated accordingly.
#
# 03.12.2007:
# I introduced Darwin_PPC and Darwin_INTEL systems; the old
# 'Darwin' was removed. This was necessary as the Intel and PPC
# MACS come with different byte swap for floating point numbers.
#
# 06.07.2008:
# Cosmetic changes to avoid error messages during install
#
# 21.12.2008:
# - The interpreters for python and perl are defined now
#   also via variables. The main problem was that 'alias'
#   deinitions were not taken into account in pipeline
#   script calls leading to problems when the script 
#   interpreters were privately installed.
# - I updated the documentation

##
## stuff that has to be modified by the user
##

# needed directories (absolutely necessary)
#
# PD is the main directory with the pipeline sources,
# i.e. the .../theli_xxxx directory. PD has to contain the
# complete, absolute path. 
#
# Make a 'good guess' for PD. DO NOT EDIT !!
DIR=`pwd`
cd ..
PD=`pwd`
cd ${DIR}

# needed programs/scripts not included in the
# pipeline distribution (absolutely necessary)
#
# The path for the bash shell (you need version
# 2.0 or higher)
P_BASH=`which bash 2> /dev/null`
#
# The path to the GNU awk program (awk is NOT sufficient)
P_GAWK=`which gawk 2> /dev/null`
#
# The path of the UNIX find program (the '-maxdepth' option
# has to be supported; use for instance the GNU find !!)
P_FIND=`which find 2> /dev/null`
#
# The path of the UNIX sort program (the '-g' option
# has to be supported; use for instance the GNU sort !!)
P_SORT=`which sort 2> /dev/null`
#
# The path to the perl and python interpreters called
# by pipeline scripts:
P_PYTHON=`which python 2> /dev/null`
P_PERL=`which perl 2> /dev/null`

#
# the following programs are not absolutely
# necessary (see the manual to learn what restrictions
# you have in this case)
#
#
# Path to the Super Mongo (sm) program
P_SM=`which sm 2> /dev/null`
#
# Path to the scamp program. The current THELI
# pipeline supports V1.3.11 of that program
#
P_SCAMP=`which scamp 2> /dev/null`
#
# Path to the aclient program. It is mandatory
# if scamp is used within THELI
P_ACLIENT=`which aclient 2> /dev/null`


# Temporary directory for small files created during the
# coaddition process. Giving '.' will put all these files
# into the reduction directory.
TEMPDIR=.

#
#
# C-compiler used in this script
# (this affects ONLY this script; not the
# C-compiler used in the compilation of the sources)
CC=gcc

#
# known architectures for source compilation (see the documentation
# for further information)
KNOWNARCH="AIX Darwin_INTEL Darwin_PPC HP-UX IRIX64 Linux Linux_64 OSF1 SunOS"

##
## here the stuff that should ever be edited by a regular user ends!!!
##


#
# Pipeline version
PIPEVERS="THELI Pipeline Version: 0.12.36"


#
# subprogram definitions
#

#
# by default the architecture is not known !!
KNOWN=0
MACHINE=""

function updatescripts() {
  DIR=`pwd`
  cd $1
  #
  # deal with the bash shell
  # scripts where we substitute the
  # first line by the actual location
  # of the bash shell
  SCRIPTFILES=`find . -name \*.sh.raw`

  for SCRIPT in ${SCRIPTFILES}
  do
    BASE=`basename ${SCRIPT} .sh.raw`
    sed -e 's!/bin/bash!'${P_BASH}'!' \
           ${SCRIPT} > ${PD}/scripts/${MACHINE}/${BASE}.sh	  
    chmod a+x ${PD}/scripts/${MACHINE}/${BASE}.sh
  done

  cd ${DIR}
}

#
# test the machine type
# run also sysconf from eclipse
#
function testmachine() {
  # first test if the endian program
  # is alreay available; if not, compile
  # it!
  if [ ! -f endian.c ]; then
    echo "need the source file 'endian.c'"
    exit 1
  else
    ${CC:?} -o endian endian.c	 
  fi

  echo "Detect your architecture:"
  echo ""

  MACHINE=`uname`
  #
  # test for LINUXALPHA architecture if
  # the machine is Linux
  if [ "${MACHINE}" = "Linux" ]; then
    LONG=`./endian | ${P_GAWK:?} '($1=="Sizes:") {print substr($5,3,1)}'`

    if [ "${LONG}" = "8" ]; then
      MACHINE="Linux_64"
    fi
  fi

  #
  # test for the different MAC/Darwin systems
  # (PPC or Intel)
  if [ "${MACHINE}" = "Darwin" ]; then
    INTEL=`./endian | ${P_GAWK:?} '($2=="machine") {print $5}'`

    MACHINE="Darwin_PPC"
    if [ "${INTEL}" = "Intel" ]; then
      MACHINE="Darwin_INTEL"
    fi
  fi

  #
  # test for known architecture
  for arch in ${KNOWNARCH}
  do
    if [ "${MACHINE}" = "${arch}" ]; then
	KNOWN=1
    fi 
  done    

  if [ "${KNOWN}" = "0" ]; then
    echo "Your machinetype \"${MACHINE}\" has not yet been tested !!"
    echo "Please refer to the manual how to proceed"
    echo ""
    ENDIAN=`./endian |  ${P_GAWK:?}'($1 == "this") {print $5}'`

    if [ "${ENDIAN}" = "Intel" ]; then
       echo "You need byte swap !!! (-DBSWAP flag)"
    else
       echo "You do not need byte swap !!!"
    fi

    LONG=`./endian | ${P_GAWK:?}  '($1=="Sizes:") {print substr($5,3,1)}'`

    if [ "${LONG}" = "8" ]; then
       echo "Long is represented by 8 bytes (-DINT64 flag)"
    else
       echo "Long is represented by 4 bytes"
    fi
    exit 1
  else
    echo "Your are running a \"${MACHINE}\" machine."
    echo "The compilation of THELI sources should work without problems"
    echo ""
  fi
  #
  # running eclipse's system configuration:
  #

  echo "---------------------------------------------------"
  echo ""
  echo "Run eclipse system configuration tool:"
  echo ""

  if [ ! -f sysconf.c ]; then
    echo "need the source file 'sysconf.c'"
    exit 1
  else
    ${CC:?} -o sysconf sysconf.c	 
  fi
  ./sysconf

  cp ./config.h ${PD}/eclipsetools/eclipse/include
  mv ./config.h ${PD}/eclipsetools/qfits

  #
  # do a simple check whether the necessary UNIX programs defined above are at 
  # least present and executable. Passing this test DOES NOT ensure that the
  # programs have the required functionality, though.

  echo ""
  echo "---------------------------------------------------"
  echo ""
  echo "Checking the presence of the programs bash, gawk, find, sort and sm:"
  echo ""

  NEEDED_PROGRAMS="P_BASH P_GAWK P_FIND P_SORT"
  OPTIONAL_PROGRAMS="P_SM P_SCAMP P_ACLIENT"
  
  for PROGRAM in ${NEEDED_PROGRAMS}
  do
    if [ "A_${!PROGRAM}" = "A_" ] || [ ! -x ${!PROGRAM} ]; then
      echo "Program defined in ${PROGRAM} not present or not executable"
      echo "Please correct this error before continuing !!"
      echo ""
      
      exit 1
    fi
  done

  echo "bash, gawk, find and sort seem to be present. This test does not"
  echo "ensure that they fulfill the criteria documented in install.sh"
  echo ""

  for PROGRAM in ${OPTIONAL_PROGRAMS}
  do
    if [ "A_${!PROGRAM}" = "A_" ] || [ ! -x ${!PROGRAM} ]; then
      echo "Program defined in ${PROGRAM} not present or not executable"
      echo "You can still use the THELI pipeline but with limited functionality !!"
      echo ""
    else
      echo "Program defined in ${PROGRAM} is present. The test does not ensure that the"
      echo "executable represents the correct program."  
    fi
  done
}

#
# create setup script responsible for setting
# the pipeline environment variables:
#
function environment() {
  # just to be sure:
  # first test the machine type if necessary
  if [ "${MACHINE}" = "" ]; then
      testmachine
  fi

  # create the pipe_env scripts
  
  sed -e 's!PIPESOFT=!PIPESOFT='${PD}'!' \
      -e 's!EXT=!EXT='${MACHINE}'!' \
      ${PD}/pipesetup/pipe_env.bash.raw \
      > ${PD}/pipesetup/pipe_env.bash.${MACHINE}
  
  sed -e 's! PIPESOFT! PIPESOFT '${PD}'!' \
      -e 's! EXT! EXT '${MACHINE}'!' \
      ${PD}/pipesetup/pipe_env.csh.raw \
      > ${PD}/pipesetup/pipe_env.csh.${MACHINE}
}

#
# print script help message:
function helpmessage() {
  echo "Install script for the THELI core pipeline"
  echo ""
  echo "This script executes necessary steps to compile the THELI sources and"
  echo "to prepare the THELI reduction scripts on your system. Before"
  echo "continuing, make sure the following preconditions:"
  echo ""
  echo "- Install and verify the functionality of the external WIFIX"
  echo "  package (a special THELI version is shipped as a separate"
  echo "  package; original sources from M. Radovich can be found at"
  echo "  http://www.na.astro.it/~radovich/wifix.htm)."
  echo "  It is not required for continuing with the package"
  echo "  installation and it is not necessary if you want to use"
  echo "  Emmanuel Bertins scamp for astrometric calibration."
  echo "  See below and the THELI documentation for further"
  echo "  information."
  echo ""
  echo "- Edit the first part of this installation script and ensure that the"
  echo "  variables P_BASH, P_PERL, P_PYTHON, P_GAWK, P_FIND and P_SORT point"
  echo "  to the required programs. They represent the absolute paths to the"
  echo "  'bash shell', the perl and python interpreters and the UNIX programs"
  echo "  gawk, find and sort.  Check that the implementations on your"
  echo "  machines fulfill the requirements documented in this script. This"
  echo "  step is essential for the use of the whole pipeline.  A similar"
  echo "  variable, P_SM exists for the external Super Mongo plotting"
  echo "  package. If you do not have this, not freely distributable, program,"
  echo "  some THELI checkplot scripts will not work but the pipeline has full"
  echo "  image processing functionality (See the THELI documentation for"
  echo "  further information)."
  echo ""
  echo "- Alternatively, or complementary, to the WIFIX package THELI supports"
  echo "  Emmanuel Bertins scamp program (currently THELI supports V1.4.6)"
  echo "  to perform astrometric calibration."
  echo "  See http://terapix.iap.fr/soft/scamp for further information."
  echo "  If you want to use this program you have to install it externally"
  echo "  together with the cdsclient to retrieve astrometric standardstar"
  echo "  catalogues via the WWW (http://vizier.u-strasbg.fr/doc/catstd-4.htx)."
  echo "  If WIFIX is installed, cdsclient does not need to be installed anymore."
  echo "  Set the variables P_SCAMP and P_ACLIENT to appropriate executables."
  echo "  Moreover, THELI scamp astrometry requires Python and the pyfits"
  echo "  module installed on your nachine"
  echo "  (see http://www.stsci.edu/resources/software_hardware/pyfits)."
  echo ""
  echo "- make sure you have the GNU C-compiler (gcc) working on your system"
  echo "  (See the THELI documentation if you would like to use a different,"
  echo "  ANSI compliant, C compiler)."
  echo ""
  echo "The THELI pipeline was previously compiled on the following UNIX "
  echo "architectures:"
  echo "Linux (32-bit and 64-bit systems), SunOS, IRIX64, OSF1, HP-UX, Darwin "
  echo "(MAC OS) and IBM-AIX. If your system is not amongst them, please consult"
  echo "the THELI documentation on how to continue. If you do not know your"
  echo "system, run the 'testmode' described below."
  echo ""
  echo "The compilation and script preparation phases are done with the"
  echo "following calls of this script:"
  echo ""
  echo '- ./install.sh -m "TEST"'
  echo "  This tests your architecture and reports whether THELI can be"
  echo "  compiled without further preparations."
  echo ""
  echo '- ./install.sh -m "ALL"'
  echo "  This compiles the THELI sources and adaptes the pipeline scripts to"
  echo "  your system. It is a combination of the COMPILE and UPDATE modes"
  echo "  and represents the complete task of this script."
  echo ""
  echo '- ./install.sh -m "COMPILE"'
  echo "  This compiles THELI sources. After compilation executables can be found"
  echo "  in .../theli-X.XX.XX/bin/MACHINE where MACHINE is your computer"
  echo "  architecture."
  echo ""
  echo '- ./install.sh -m "UPDATE"'
  echo "  This adapts the pipeline scripts to your system. The modified and "
  echo "  ready-to-use scripts can be found in .../theli-X.XX.XX/scripts/MACHINE"
  echo ""
  echo "Consult the THELI documentation for further information on the"
  echo "installation process."
}


# read command line argument(s)

if [ $# -lt 2 ]; then
  helpmessage
  exit 1
fi

# 4 MODES available:
# - TEST: the the machine type
# - ALL: do script update and compile sources
# - UPDATE: do script update
# - COMPILE: compile sources

MODE=""
GoOn=0
while [ $GoOn = 0 ]
do
  case $1 in
  -m)
      MODE=${2}
      shift
      shift
      ;;
   *)
      GoOn=1
      ;;
  esac
done

#
# test machine type
#
for mode in ${MODE}
do
  if [ "${mode}" = "TEST" ]; then
     testmachine
  fi    
done

#
# update scripts
#
for mode in ${MODE}
do
  if [ "${mode}" = "UPDATE" ] || [ "${mode}" = "ALL" ]; then

    # first test the machine type if necessary
    if [ "${MACHINE}" = "" ]; then
	testmachine
    fi

    #
    # create machine dependent subdirectory in the
    # scripts directory
    if [ ! -d ${PD}/scripts/${MACHINE} ]; then
	mkdir ${PD}/scripts/${MACHINE}
    fi

    # create setup script for pipeline environment
    # if necessary:
    if [ ! -f ${PD}/pipesetup/pipe_env.bash.${MACHINE} ]; then
      environment
    fi
    
    # now the progs.ini file
    
    sed -e 's!PIPESOFT=!PIPESOFT='${PD}'!' \
        -e 's!^PHOTSTANDARDSTARSCAT=!PHOTSTANDARDSTARSCAT='${PHOTSTANDARDSTARCAT}'!' \
        -e 's!^STANDARDSTARSCAT=!STANDARDSTARSCAT='${STANDARDSTARCAT}'!' \
        -e 's!TEMPDIR=.!TEMPDIR='${TEMPDIR}'!' \
        -e 's!BIN=${PIPESOFT}/bin/!BIN=${PIPESOFT}/bin/'${MACHINE}'/!' \
        -e 's!UTILSCRIPTS=${PIPESOFT}/scripts/utilities!UTILSCRIPTS=${PIPESOFT}/scripts/'${MACHINE}'/!' \
        -e 's!P_PYTHON=!P_PYTHON='${P_PYTHON}'!' \
        -e 's!P_PERL=!P_PERL='${P_PERL}'!' \
        -e 's!P_GAWK=!P_GAWK='${P_GAWK}'!' \
        -e 's!P_FIND=!P_FIND='${P_FIND}'!' \
        -e 's!P_SORT=!P_SORT='${P_SORT}'!' \
        -e 's!P_SM=!P_SM='${P_SM}'!' \
        -e 's!P_SCAMP=!P_SCAMP='${P_SCAMP}'!' \
        -e 's!P_ACLIENT=!P_ACLIENT='${P_ACLIENT}'!' \
        -e 's!S_ASTROMETRIX=!S_ASTROMETRIX='${PD}'/scripts/astrometrix/astrom_theli!' \
        -e 's!S_PHOTOMETRIX=!S_PHOTOMETRIX='${PD}'/scripts/astrometrix/photom_theli!' \
        ${PD}/scripts/reduction/progs.ini.raw \
        > ${PD}/scripts/${MACHINE}/progs.ini
    
    # now update all the scripts
    
    SCRIPTDIRS="${PD}/scripts/reduction ${PD}/scripts/WFI_reduction
                ${PD}/scripts/reduction ${PD}/scripts/CFH12K_reduction
                ${PD}/scripts/parallel  ${PD}/scripts/utilities"
    
    for SCRIPTDIR in ${SCRIPTDIRS}
    do
      updatescripts ${SCRIPTDIR}
      #
      # just copy possible Python scripts
      # (they are called via the env command and
      # hence we do not need to hardcode the location
      # of the Python executable):
      PYFILES=`find ${SCRIPTDIR} -name \*.py`

      for pyfile in ${PYFILES}
      do
	BASE=`basename ${pyfile}`
        cp ${pyfile} ${PD}/scripts/${MACHINE}
	chmod a+x ${PD}/scripts/${MACHINE}/${BASE}
      done
      #
      # copy include files
      INCLUDE=`find ${SCRIPTDIR} -name \*.include`

      for include in ${INCLUDE}
      do
        cp ${include} ${PD}/scripts/${MACHINE}
      done
    done

    # copy files from the instrument specific directories
    # to the machine directories

    cp ${PD}/scripts/WFI_reduction/WFI* ${PD}/scripts/${MACHINE}
    cp ${PD}/scripts/CFH12K_reduction/CFH12K* ${PD}/scripts/${MACHINE}
    cp ${PD}/scripts/CFH12K_reduction/cfh12k* ${PD}/scripts/${MACHINE}
    
    chmod a+x ${PD}/pipesetup/test_dir
    chmod a+x ${PD}/pipesetup/vraag

    # now deal with the perl scripts from ASTROMETRIX:
    sed -e "s!GaBoDSVersion=!GaBoDSVersion=\"${PIPEVERS}\";!" \
      ${PD}/scripts/astrometrix/src/photom_theli.raw >\
      ${PD}/scripts/astrometrix/photom_theli
    chmod +x ${PD}/scripts/astrometrix/photom_theli
    sed -e "s!GaBoDSVersion=!GaBoDSVersion=\"${PIPEVERS}\";!" \
      ${PD}/scripts/astrometrix/src/astrom_theli.raw >\
      ${PD}/scripts/astrometrix/astrom_theli
    chmod +x ${PD}/scripts/astrometrix/astrom_theli
    sed -e "s!GaBoDSVersion=!GaBoDSVersion=\"${PIPEVERS}\";!" \
      ${PD}/scripts/astrometrix/src/nldacsub_theli.pm.raw >\
      ${PD}/scripts/astrometrix/nldacsub_theli.pm
    cp ${PD}/scripts/astrometrix/src/AstSubs_theli.pm.raw ${PD}/scripts/astrometrix/AstSubs_theli.pm
    cp ${PD}/scripts/astrometrix/src/AstConf_theli.pm.raw ${PD}/scripts/astrometrix/AstConf_theli.pm
    cp ${PD}/scripts/astrometrix/src/catsubs_theli.pm.raw ${PD}/scripts/astrometrix/catsubs_theli.pm
    cp ${PD}/scripts/astrometrix/src/PhotConf_theli.pm.raw ${PD}/scripts/astrometrix/PhotConf_theli.pm
  fi
done

# compile sources

for mode in ${MODE}
do
  if [ "${mode}" = "COMPILE" ] || [ "${mode}" = "ALL" ]; then

    # do a test first whether the machine type
    # is already known
    if [ "${MACHINE}" = "" ]; then
      testmachine
    fi

    # create setup script for pipeline environment
    # if necessary:
    if [ ! -f ${PD}/pipesetup/pipe_env.bash.${MACHINE} ]; then
      environment
    fi

    # set environment variables necessary for compilation
    . ${PD}/pipesetup/pipe_env.bash.${MACHINE}

    # write the correct global_vers.h file:
    sed -e "s!#define __PIPEVERS__!#define __PIPEVERS__ \"${PIPEVERS}\"!" \
	  ${PD}/pipesetup/global_vers.h.raw >\
	  ${PD}/ldactools/library/global_vers.h

    cd ${PD}/pipesetup/
    make all
    cd ${PD}/pipesetup/
  fi
done









